PyBEST is a *Py*thonic *B*lack-box *E*lectronic *S*tructure *T*ool,
written primarily in the [Python programming language](https://www.python.org/)
with additional parts written in C++ (interfaced using [nanobind](https://nanobind.readthedocs.io/en/latest/why.html).

PyBEST is a fully-fledged modern electronic-structure software package, developed at
[NCU in Torun](https://www.umk.pl/en/), released under the GNU General Public License.
The package provides an efficient and reliable platform for electronic structure
calculations at the interface of chemistry and physics using unique electronic structure
methods. PyBEST is easy to use and easy to code in.

For more information, see our official [homepage](http://www.fizyka.umk.pl/~pybest).
You will also find there various versions of PyBEST to download and install including
the most recent version of this documentation.


Main Features
=============

**Compatibility**

* Integrating the [LibInt2](https://github.com/evaleev/libint) library.

* Adapting a modular structure to easily combine PyBEST with custom developments.

* GPU-accelerated computing using the Cupy library

**Electronic Structure Methods**

* Hamiltonians (various one- and two-electron integrals)

    * The electronic Schrödinger Hamiltonian in atom-centered Gaussian basis sets

        * One- and two-electron integrals computed with
          [LibInt2](https://github.com/evaleev/libint)

            * overlap, kinetic energy, nuclear, multipole, electron repulsion integrals (ERI)

        * pVp integrals

        * Cholesky-decomposed ERI

        * Douglas-Kroll-Hess Hamiltonian (DKH2)

        * Point-charge integrals

        * Static embedding potential integrals

    * Model Hamiltonians

        * 1-dimensional Hubbard model Hamiltonian (with and without periodic boundary
          conditions)

        * 1-dimensional contact interactions in an arbitrary external potential

    * User-provided Hamiltonians

        * Molpro's FCIDUMP format

* Hartree-Fock theory:

    * Restricted and unrestricted orbitals

    * Various DIIS algorithms

* Configuration Interaction methods

    * Configuration Interaction Singles (CIS)

    * Configuration Interaction Doubles (CID)

    * Configuration Interaction Singles and Doubles (CISD)

* Coupled Cluster-type methods

    * pair-Coupled Cluster Doubles (pCCD), also known as the Antisymmetry Product of 1 reference-orbital Geminals (AP1roG)

    * Linearized Coupled Cluster Doubles (LCCD)

    * Linearized Coupled Cluster Singles and Doubles (LCCSD)

    * pCCD-LCCD (pCCD with an LCCD correction on top of it)

    * pCCD-LCCSD (pCCD with an LCCSD correction on top of it)

    * pCCD-CCS (pCCD with a Coupled Cluster Singles correction on top of it)

    * Coupled Cluster Doubles (CCD) in any restricted orbital basis

    * Coupled Cluster Singles and Doubles (CCSD) in any restricted orbital basis

    * Frozen-pair Coupled Cluster Doubles (fpCCD)

    * Frozen-pair Coupled Cluster Singles Doubles (fpCCSD)

    * Tailored Couples Cluster Singles Doubles (tCCSD)

* Perturbation theory

    * Moller-Plesset Perturbation Theory of second order (MP2)

    * Symmetry Adapted Perturbation Theory (SAPT)

        * the SAPT0 un-coupled flavour for spin-restricted HF orbitals

    * Various Perturbation Theory models of second order with an pCCD reference function

        * PT2SDd (diagonal zero-order Hamiltonian, single determinant as dual state) with
          Doubles and Singles and Doubles

        * PT2MDd (diagonal zero-order Hamiltonian, pCCD as dual state) with Doubles and
          Singles and Doubles

        * PT2SDo (off-diagonal zero-order Hamiltonian, single determinant as dual state)
          with Doubles and Singles and Doubles

        * PT2MDo (off-diagonal zero-order Hamiltonian, pCCD as dual state) with Doubles
          and Singles and Doubles

        * PT2b (off-diagonal zero-order Hamiltonian, pCCD as dual state, full Hamiltonian
          as perturbation operator) with Doubles and Singles and Doubles including/excluding
          electron pairs

* Equation of Motion Coupled Cluster methods

    * Equation of Motion CCS (EOM-CCS equivalent to CIS)

    * EOM-pCCD

    * EOM-pCCD+S (single excitation are included in the EOM ansatz only, while the CC ansatz is restricted to pCCD)

    * EOM-pCCD-CCS

    * EOM-LCCD

    * EOM-LCCSD

    * EOM-pCCD-LCCD

    * EOM-pCCD-LCCSD

* Ionization Potential Equation of Motion Coupled Cluster methods

    * IP-pCCD with 1 hole and 2 hole 1 particle operators ($`S_z = 0.5, 1.5`$)

    * DIP-pCCD with 2 hole and 3 hole 1 particle operators ($`S_z = 0.0, 1.0, 2.0`$)

**Post-Processing**

* Orbital entanglement and correlation analysis

    * for a pCCD reference function

    * for a pCCD-LCC reference function

* Orbital localization

    * Pipek-Mezey orbital localization

* Property calculations

    * Dipole moment with HF, pCCD, PCCD-LCC wavefunctions

    * Quadrupole moment with HF, pCCD, PCCD-LCC wavefunctions


Citing PyBEST
==============

Please, refer to PyBEST as follows:

> Saman Behjou, Filip Brzęk, Rahul Chakraborty, Kacper Cieślak, Antonina Dobrowolska, Seyedehdelaram Jahani,
> Zahra Karimi, Michał Kopczyński, Aleksandra Leszczyk, Artur Nowak, Lena Szczuczko, Emil Sujkowski,
> Julian Świerczyński, Somayeh Ahmadkhani, Katharina Boguslawski, Dariusz Kędziera, Maximilian H. Kriebel,
> Paweł Tecmer, Piotr Szymon Żuchowski, PyBEST 2.0.0, **2024**


If you use PyBEST, please cite the following:

> Katharina Boguslawski, Filip Brzęk, Rahul Chakraborty, Kacper Cieślak,
> Seyedehdelaram Jahani, Aleksandra Leszczyk, Artur Nowak, Emil Sujkowski,
> Julian Świerczyński, Somayeh Ahmadkhani, Dariusz Kędziera,
> Maximilian H. Kriebel, Piotr Szymon Żuchowski, Paweł Tecmer,
> "PyBEST: Improved functionality and enhanced performance",
> *Computer Physics Communications*, **297**, 2024, https://doi.org/10.1016/j.cpc.2023.109049.


> Katharina Boguslawski, Aleksandra Leszczyk, Artur Nowak, Filip Brzęk, Piotr Szymon Żuchowski,
> Dariusz Kędziera, Paweł Tecmer, "Pythonic Black-box Electronic Structure Tool (PyBEST).
> An open-source Python platform for electronic structure calculations at the interface between chemistry and physics",
> *Computer Physics Communications*, **264**, 2021, https://doi.org/10.1016/j.cpc.2021.107933.


Contact Information
===================

If you have questions, please send them to our developer e-mail address
**pybest@umk.pl**

Alternatively, you may directly contact the authors of a specific part of
PyBEST.
