// PyBEST: Pythonic Black-box Electronic Structure Tool
// Copyright (C) 2016-- The PyBEST Development Team
//
// This file is part of PyBEST.
//
// PyBEST is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// PyBEST is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>
// --
#include "cholesky_eri.h"

// this is just minimal wrapper unpacking PYBEST's PyBasis to libint2::BasisSet
// which is default
nb::ndarray<nb::numpy, double> py_ints::compute_cholesky_eri(
    Basis *basis0, double cd_threshold, size_t cd_beta) {
#ifndef LIBCHOL_AVAILABLE
  throw std::runtime_error(
      "PyBEST was not compiled against the Libchol module!");
#endif
#ifdef LIBCHOL_AVAILABLE
  // C++ generated std::vector<double> into nb:ndarray<double> with capsule to
  // guard de-allocation
  // https://nanobind.readthedocs.io/en/latest/ndarray.html#binding-functions-that-return-arrays
  std::vector<double> *v = new std::vector<double>(
      chol::compute_cholesky_decomposition(basis0->obs, cd_threshold, cd_beta));
  nb::capsule deleter(v, [](void *p) noexcept {
    delete reinterpret_cast<std::vector<double> *>(p);
  });
  std::size_t shape[1] = {reinterpret_cast<std::size_t>(v->size())};
  return nb::ndarray<nb::numpy, double>(v->data(), 1, shape, deleter);
#endif
}
