
## Generating stubs for `core.so` c-extension

1. Build pybest locally with the following command to obtain `core.so`, within src folder

    ```shell
    PYBEST_ENABLE_CHOLESKY=1 python3 -m pip install --no-build-isolation -e .
    ```

1. Install `pybind11-stubs` and run:

    ```shell
    pybind11-stubgen pybest.core.core -o src/pybest/core
    ```

1. This creates most of the stubs automatically, a few things to fix is about NDarray stubs:

    Based on `https://stackoverflow.com/a/75153042`, a type we seek was `NDArray[np.float64]`

    ```python
    import numpy as np
    from numpy.typing import NDArray

    NDArray[np.float64]

    ```

1. (after installing pybest) Run mypy src/pybest/core src/pybest/gbasis. Observe any issues & fix them.

1. Voila, we have consistent types for `core.so` c-extension, ready for consumption in the rest of the code
