# PyBEST: Pythonic Black-box Electronic Structure Tool
# Copyright (C) 2016-- The PyBEST Development Team
#
# This file is part of PyBEST.
#
# PyBEST is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# PyBEST is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
# --
"""Module ints to calculate various integrals using libint2"""

from __future__ import annotations

import numpy as np
from numpy.typing import NDArray

from . import Basis, ExternalCharges

__all__ = [
    "compute_cholesky_eri_cpp",
    "compute_dipole_cpp",
    "compute_eri_cpp",
    "compute_external_charges_cpp",
    "compute_kinetic_cpp",
    "compute_nuclear_cpp",
    "compute_nuclear_pc_cpp",
    "compute_nuclear_repulsion_cpp",
    "compute_overlap_cpp",
    "compute_point_charges_cpp",
    "compute_ppcp_cpp",
    "compute_pvp_cpp",
    "compute_quadrupole_cpp",
]

def compute_cholesky_eri_cpp(
    basis0: Basis, cd_threshold: float, cd_beta: int
) -> NDArray[np.float64]:
    """A function to cholesky decomposed compute the electron repulsion integrals in the AO basis"""

def compute_dipole_cpp(
    basis0: Basis,
    basis1: Basis,
    olp: NDArray[np.float64],
    mux: NDArray[np.float64],
    muy: NDArray[np.float64],
    muz: NDArray[np.float64],
    x: float,
    y: float,
    z: float,
) -> None:
    """A function to compute the dipole integrals in the AO basis"""

def compute_eri_cpp(
    obs0: Basis,
    obs1: Basis,
    obs2: Basis,
    obs3: Basis,
    out: NDArray[np.float64],
    symmetry: bool,
) -> None:
    """A function to compute the electron repulsion integrals in the AO basis"""

def compute_external_charges_cpp(ext_charges: ExternalCharges) -> float:
    """A function to compute the potential energy due to the interaction  between point charges"""

def compute_kinetic_cpp(
    obs0: Basis, obs1: Basis, out: NDArray[np.float64], uncontract: bool
) -> None:
    """A function to compute the kinetic energy integrals in the AO basis"""

def compute_nuclear_cpp(
    obs0: Basis, obs1: Basis, out: NDArray[np.float64], uncontract: bool
) -> None:
    """A function to compute the electron-nuclei attraction integrals in the AO basis"""

def compute_nuclear_pc_cpp(
    basis0: Basis, ext_charges: ExternalCharges
) -> float:
    """A function to compute the potential energy due to the interaction of the external point charges and the nuclei"""

def compute_nuclear_repulsion_cpp(basis0: Basis, basis1: Basis) -> float:
    """A function to compute the potential energy due to nuclear-nuclear repulsion"""

def compute_overlap_cpp(
    basis0: Basis, basis1: Basis, output: NDArray[np.float64], uncontract: bool
) -> None:
    """A function to compute the AO overlap"""

def compute_point_charges_cpp(
    obs0: Basis,
    obs1: Basis,
    charges: ExternalCharges,
    out: NDArray[np.float64],
    uncontract: bool,
) -> None:
    """A function that computes the interaction between external point charges and electrons"""

def compute_ppcp_cpp(
    obs0: Basis,
    obs1: Basis,
    charges: ExternalCharges,
    out: NDArray[np.float64],
    uncontract: bool,
) -> None:
    """A function to compute the pPCp integrals in the uncontracted AO basis"""

def compute_pvp_cpp(
    obs0: Basis, obs1: Basis, charges: NDArray[np.float64], uncontract: bool
) -> None:
    """A function to compute the pVp integrals in the uncontracted AO basis"""

def compute_quadrupole_cpp(
    basis0: Basis,
    basis1: Basis,
    olp: NDArray[np.float64],
    mux: NDArray[np.float64],
    muy: NDArray[np.float64],
    muz: NDArray[np.float64],
    muxx: NDArray[np.float64],
    muxy: NDArray[np.float64],
    muxz: NDArray[np.float64],
    muyy: NDArray[np.float64],
    muyz: NDArray[np.float64],
    muzz: NDArray[np.float64],
    x: float,
    y: float,
    z: float,
) -> None:
    """A function to compute the quadrupole integrals in the AO basis"""
