// PyBEST: Pythonic Black-box Electronic Structure Tool
// Copyright (C) 2016-- The PyBEST Development Team
//
// This file is part of PyBEST.
//
// PyBEST is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// PyBEST is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>
// --

#include "sapt_utils.h"

void py_sapt_core::get_sapt_amplitudes(
    nb::ndarray<nb::numpy, double> t_rsab_npy,
    nb::ndarray<nb::numpy, double> t_ra_npy,
    nb::ndarray<nb::numpy, double> t_sb_npy,
    nb::ndarray<nb::numpy, double> en_occ_a_npy,
    nb::ndarray<nb::numpy, double> en_virt_a_npy,
    nb::ndarray<nb::numpy, double> en_occ_b_npy,
    nb::ndarray<nb::numpy, double> en_virt_b_npy, std::size_t nocc_a,
    std::size_t nocc_b, std::size_t nvirt_a, std::size_t nvirt_b) {
  // pybind11 boilerplate
  double *t_rsab = (double *)t_rsab_npy.data();
  double *t_ra = (double *)t_ra_npy.data();
  double *t_sb = (double *)t_sb_npy.data();
  double *en_occ_a = (double *)en_occ_a_npy.data();
  double *en_virt_a = (double *)en_virt_a_npy.data();
  double *en_occ_b = (double *)en_occ_b_npy.data();
  double *en_virt_b = (double *)en_virt_b_npy.data();

  double tmp_1 = 0, tmp_2 = 0;
  // TODO: change occupy idx to be most-outer one, requires reordering of the
  // tensors.
  for (std::size_t r = 0; r < nvirt_a; r++) {
    for (std::size_t a = 0; a < nocc_a; a++) {
      tmp_1 = en_occ_a[a] - en_virt_a[r];
      // T1 - mon a induction denominator
      t_ra[r * nocc_a + a] = 1.0 / tmp_1;
      for (std::size_t s = 0; s < nvirt_b; s++) {
        for (std::size_t b = 0; b < nocc_b; b++) {
          tmp_2 = en_occ_b[b] - en_virt_b[s] + tmp_1;
          // T2 - ABAB dispersion denominator
          t_rsab[((r * nvirt_b + s) * nocc_a + a) * nocc_b + b] = 1.0 / tmp_2;
        }
      }
    }
  }
  for (std::size_t s = 0; s < nvirt_b; s++) {
    for (std::size_t b = 0; b < nocc_b; b++) {
      tmp_2 = en_occ_b[b] - en_virt_b[s];
      // T1 - mon b induction denominator
      t_sb[s * nocc_b + b] = 1.0 / tmp_2;
    }
  }
}
